<?php

namespace Simpler\Services;

use Simpler\Models\OrderRequest;
use Simpler\Exceptions\InvalidProductException;
use Simpler\Exceptions\InvalidOrderException;

final class OrderService
{

    /**
     * @throws InvalidProductException
     * @throws \WC_Data_Exception
     */
    public function create_order($user_id, OrderRequest $order_request)
    {
        $existing = $this->get_order_by_meta('simpler_cart_id', $order_request->get_order()->get_simpler_cart_id());
        if (count($existing) > 0) {
            return $existing[0];
        }

        $wc_order = new \WC_Order(0);

        $wc_order->set_customer_id($user_id);
        $wc_order->update_meta_data(
            'simpler_cart_id',
            $order_request->get_order()->get_simpler_cart_id()
        );

        $wc_order = $this->create_line_items($wc_order, $order_request->get_order()->get_cart());
        $wc_order = $this->assign_shipping_address($wc_order, $order_request);

        $wc_order = $this->create_shipping_line_item(
            $wc_order,
            $order_request->get_order()->get_shipping(),
            $order_request->get_order()->get_shipping_method_id()
        );

        $wc_order = $this->apply_coupon_if_present($wc_order, $order_request->get_order()->get_coupon());
        $wc_order->set_payment_method('Simpler');
        $wc_order->set_payment_method_title('Simpler');
        $wc_order->calculate_totals();
        $wc_order->set_status(\apply_filters('woocommerce_default_order_status', 'processing'));
        $wc_order->save();

        return $wc_order;
    }

    private function get_order_by_meta($key, $val)
    {
        return \wc_get_orders(array(
            'limit'      => 1,
            'meta_key'   => $key,
            'meta_value' => $val,
        ));
    }

    /**
     * @throws InvalidProductException
     */
    private function create_line_items($wc_order, $line_items)
    {
        foreach ($line_items as $item) {
            $product = \wc_get_product($item->get_product_id());
            if (!$product || \is_wp_error($product)) {
                error_log('Could not retrieve product with id=' . $item->get_product_id());
                $wc_order->delete();
                throw new InvalidProductException();
            }
            $wc_order->add_product($product, $item->get_quantity());
        }

        return $wc_order;
    }

    private function assign_shipping_address($wc_order, OrderRequest $request)
    {
        $address = $request->get_ship_to();
        $user    = $request->get_user();
        //shipping
        if ($address) {
            $wc_order->set_shipping_first_name($user->get_first_name());
            $wc_order->set_shipping_last_name($user->get_last_name());
            $wc_order->set_shipping_address_1($address->get_address());
            $wc_order->set_shipping_address_2($address->get_notes());
            $wc_order->set_shipping_city($address->get_city());
            $wc_order->set_shipping_postcode($address->get_postcode());
            $wc_order->set_shipping_state($address->get_state());
            $wc_order->set_shipping_country($address->get_country());
        }

        //billing
        $wc_order->set_billing_first_name($user->get_first_name());
        $wc_order->set_billing_last_name($user->get_last_name());
        if ($address) {
            $wc_order->set_billing_phone($address->get_phone());
            $wc_order->set_billing_address_1($address->get_address());
            $wc_order->set_billing_address_2($address->get_notes());
            $wc_order->set_billing_city($address->get_city());
            $wc_order->set_billing_postcode($address->get_postcode());
            $wc_order->set_billing_state($address->get_state());
            $wc_order->set_billing_country($address->get_country());
        }

        return $wc_order;
    }

    /**
     * @throws \WC_Data_Exception
     */
    private function create_shipping_line_item($wc_order, $shipping, $method_id)
    {
        if (is_null($shipping) && !$method_id) {
            return $wc_order; //nothing to do
        }
        $line_item = new \WC_Order_Item_Shipping();
        if (is_null($shipping)) {
            $method = $this->get_shipping_method_instance($method_id);
            $line_item->set_instance_id($method->get_instance_id());
            $line_item->set_method_id($method->id);
            $line_item->set_method_title($method->get_method_title());
            $line_item->set_total($method->get_instance_option('cost'));
        } else {
            $line_item->set_instance_id($shipping->get_instance_id());
            $line_item->set_method_id($shipping->get_method_id());
            $line_item->set_method_title($shipping->get_method_title());
            $line_item->set_total(\wc_format_decimal($shipping->get_cost()));
        }

        $wc_order->add_item($line_item);
        return $wc_order;
    }

    private function get_shipping_method_instance($method_id)
    {
        if (!$method_id) {
            throw new InvalidOrderException("no method id received");
        }

        $method_id_num = intval($method_id);
        foreach (\WC_Shipping_Zones::get_zones() as $zone) {
            foreach ($zone['shipping_methods'] as $method) {
                if ($method->get_instance_id() === $method_id_num) {
                    return $method;
                }
            }
        }

        foreach (WC()->shipping->get_shipping_methods() as $method) {
            if ($method->id === $method_id) {
                return $method;
            }
        }

        throw new InvalidOrderException("did not find shipping method " . $method_id);
    }

    private function apply_coupon_if_present($wc_order, $coupon)
    {
        if ($coupon !== NULL) {
            $wc_order->apply_coupon($coupon);
        }
        return $wc_order;
    }
}
